#!/usr/bin/perl -w
# vim: set sw=4 ts=4 si et:
# Simple single threaded web server written in perl.
# It provides a web page where one can shutdown the server remotely. 
# This is intendent to be used only internally in trusted LAN environments
# were security is not a concern.
# Author: Guido Socher
#
# Usage: shutdown-button-http.pl [-p portnumber] [-i ip-to-bind-to] 
# Example: ./shutdown-button-http.pl -i 192.168.0.3
# The default port number is 88.
# Try:
# curl -v http://192.168.0.3:88
#
sub help();
sub greeter();
use vars qw($opt_h $opt_p $opt_i);
use Getopt::Std;
use strict;
use Socket;
getopts("hp:i:")||die "ERROR: No such option. -h for help.\n";
help() if($opt_h);
help() if($ARGV[0]);
# === customize this section:
my $pw="sec";  # the password that the user must specify to trigger a shutdown.
my $listenport = 88;
# === end customize, do not touch anything below.
#
$listenport=$opt_p if ($opt_p);
socket (Server, PF_INET, SOCK_STREAM, 6) || die ("Error socket: $!"); # protocol 6 is tcp
setsockopt(Server, SOL_SOCKET,SO_REUSEADDR,1) || die ("Error setsockopt: $!");
my $sockaddr = sockaddr_in($listenport, $opt_i ? inet_aton($opt_i) : INADDR_ANY) || die ("Error sockaddr_in: $!");
bind(Server,$sockaddr) || die ("Error bind: $!");
listen(Server,SOMAXCONN) || die ("Error listen: $!");

my $caddr;
my $buffer;
while ($caddr = accept(Client, Server)) {
    recv(Client,$buffer,1200,0);
    #print "dbg client:\n$buffer\n";
    if ($buffer && $buffer=~/^GET /){
        print Client "HTTP/1.1 200 OK\r\n";
        print Client "Content-Type: text/html\r\n";
        print Client "Server: shutdown-button-http/0.1\r\n";
        print Client "Connection: close\r\n";
        print Client "\r\n";
        print Client greeter();
    }
    # for a post you get something like this:
    #POST / HTTP/1.1
    #Host: localhost:8000
    #User-Agent: Mozilla/5.0 (X11; Linux x86_64; rv:58.0) Gecko/20100101 Firefox/58.0
    #Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8
    #Accept-Language: en-US,en;q=0.5
    #Accept-Encoding: gzip, deflate
    #Content-Type: application/x-www-form-urlencoded
    #Content-Length: 21
    #Referer: http://lenie:8000/
    #Connection: keep-alive
    #Upgrade-Insecure-Requests: 1
    #
    #pw=ls&action=shutdown
    if ($buffer && $buffer=~/^POST /){
		my %FORM;
        my $contentlen=0;
        my $readcontent=0;
        my $pbuf="";
        for my $line (split(/\r\n/,$buffer)){
            if ($line =~/Content-Length: (\d+)/){
                $contentlen=int($1);
            }
            if ($line eq ""){
                $readcontent=1;
                next;
            }
            if ($readcontent){
                $pbuf=$line;
            }
        }
        foreach my $pair (split(/\&/, $pbuf)){
            my ($name, $value) = split(/=/, $pair);
            $value = "" unless (defined $value);
            # Un-Webify plus signs and %-encoding
            $value =~ tr/+/ /;
            $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
            $FORM{$name} = $value;
        }
        print Client "HTTP/1.1 200 OK\r\n";
        print Client "Content-Type: text/html\r\n";
        print Client "Server: shutdown-button-http/0.1\r\n";
        print Client "Connection: close\r\n";
        print Client "\r\n";
        if ($FORM{"action"} && $FORM{"action"} eq "shutdown"){
            if ($FORM{"pw"} && $FORM{"pw"} eq $pw){
                print Client "<h2>OK</h2>\n<pre>shutdown ...\n";
                # run the command and capture the output:
                #my $cmdstatus=`/usr/bin/sudo -n /sbin/shutdown -h now 2>&1`;
                my $cmdstatus=`/sbin/shutdown -h now 2>&1`;
                print Client "\n$cmdstatus\n";
                print Client "</pre><br><hr>\n";
            }else{
                print Client "<h2>ERROR</h2>\n<p>Authentication error.</p>\n";
            }
        }else{
            print Client "<h2>ERROR</h2>\n<p>Request error</p>\n";
        }
    }
    close Client;
}
#
sub help(){
print "shutdown-button-http.pl -- a simple web server to provide a web page with a server shutdown button

USAGE: shutdown-button-http.pl [-h] [-p portnumber] [-i ip-to-bind-to] 

This is a simple single threaded web server written in perl (no non standard module dependency)
to display a shutdown button.

OPTIONS: -h this help
         -i specify the ip address of a local interface to bind to
         -p portnumber (default is 88)

EXAMPLE: shutdown-button-http.pl -p 8000 -i 127.0.0.1
\n";
exit;
}

sub greeter(){
    my $h=`hostname`;
    chomp($h);
return qq|<HTML>
<HEAD>
   <TITLE>$h, shutdown</TITLE>
</HEAD>
<BODY>
<CENTER>
<h2>$h, shutdown</h2>
<FORM METHOD="post" ACTION="/"> 
    <TABLE BORDER="1" CELLPADDING="5"cellspacing="0" bgcolor="#f0ff10">
    <TR>
    <TD ALIGN="center">
        Password: &nbsp;
        <INPUT TYPE="password" NAME="pw" SIZE=10 VALUE="">
        <INPUT TYPE="hidden" NAME="action" VALUE="shutdown">
        <br>
        <br>
          <input type=submit VALUE=" OK ">
    </td>
    </tr>
    </table>
</form>
<br>
</CENTER>
<hr>
</body>
</html>
|;
}
#--------------------------------------------------------
__END__ 
